/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_POP_IPC_TEST_H
#define CUBELIB_POP_IPC_TEST_H

#include <string>
#include "PerformanceTest.h"


namespace popcalculation
{
class POPIPCTest : public PerformanceTest
{
private:
    cube::Metric* pop_ipc;
    double        maxValue;


protected:

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false   );


    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "IPC Efficiency compares IPC to the reference, where lower values indicate that \n"
            "rate of computation has slowed. Typical causes for this include decreasing cache hit rate \n"
            "and exhaustion of memory bandwidth, these can leave processes stalled and waiting for data. \n";
        std::string not_active_text =
            "IPC Efficiency metric is available only, if the measurement has collected \n"
            "either the PAPI counters PAPI_TOT_INS and PAPI_TOT_CYC, \n"
            "or the Perf counters instructions and cycles.\n"
            "How to do it see Score-P manual, PERF COUNTERS";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;

public:
    POPIPCTest( cube::CubeProxy* );

    virtual
    ~POPIPCTest()
    {
    };
// ------ overview tests ---------

    virtual
    inline
    double
    getMaximum() const
    {
        return maxValue;
    }

    virtual
    inline
    bool
    isPercent() const
    {
        return false;
    }

    bool
    isActive() const;

    bool
    isIssue() const;

    virtual
    inline
    bool
    isNormalized() const // returns true, if range goes from 0 to 1. false -> otherwise
    {
        return false;
    }
};
};
#endif // POP_ipc_TEST_H
