/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_POP_STALLED_RESOURCES_TEST_H
#define CUBELIB_POP_STALLED_RESOURCES_TEST_H

#include <string>
#include "PerformanceTest.h"


namespace popcalculation
{
class POPStalledResourcesTest : public PerformanceTest
{
private:
    cube::Metric* pop_stalled_resources;


protected:

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false   );

    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "Stalled resources indicates the fraction of computational cycles \n"
            "in user code that the processor has been waiting for some resources.\n"
            "If PAPI counters are available, use PAPI_RES_STL / PAPI_TOT_CYC. \n"
            "Otherwise, if Perf counters are available, use one of \n"
            "\t(stalled-cycles-backend+ stalled-cycles-frontend) / cycles\n"
            "\tstalled-cycles-backend / cycles\n"
            "\tstalled-cycles-frontend / cycles \n"
            "depending on what is available.\n";

        std::string not_active_text =
            "Stalled resources (only computation) is available only, if the \n"
            "measurement has collected either \n"
            "the PAPI counters PAPI_TOT_INS, \n"
            "or the Perf counters instructions and cycles.\n"
            "How to do it see Score-P manual, PERF COUNTERS";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;

public:
    POPStalledResourcesTest( cube::CubeProxy* );

    virtual
    ~POPStalledResourcesTest()
    {
    };
// ------ overview tests ---------

    bool
    isActive() const;

    bool
    isIssue() const;

    virtual
    inline
    bool
    isRegular() const // returns true, if 0 to 1 indicates improvement, false - otherwise
    {
        return false;
    }
};
};
#endif // POP_stalled_resources_TEST_H
