/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_POP_HYBRID_TRANSFER_TEST_ADD_H
#define CUBELIB_POP_HYBRID_TRANSFER_TEST_ADD_H

#include <string>
#include "PerformanceTest.h"


namespace hybaddanalysis
{
class POPHybridCommunicationEfficiencyTestAdd;
class POPHybridTransferTestAdd : public popcalculation::PerformanceTest
{
    friend class POPHybridCommunicationEfficiencyTestAdd;
private:
    cube::Metric* max_total_time_ideal;
    cube::Metric* max_total_time;

    cube::list_of_metrics lmax_total_time_ideal;

    void
    add_max_total_time_ideal( cube::CubeProxy* ) const;

    void
    add_max_total_time( cube::CubeProxy* ) const;

protected:

    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "Transfer Efficiency (TE) measures inefficiencies due to time spent in data transfers:\n"
            "\tTE =  maximum across processes(total runtime on ideal network )/ maximum across processes(total measured runtime ) \n\n"
            "where total run-time on ideal network is a runtime without detected by Scalasca waiting time and MPI I/O time. ";
        std::string not_active_text =
            "Transfer Efficiency metric is available only, if MPI wait states have been detected and measured. \n"
            "Hence it is only available for trace analysis results of Scalasca such as scout.cubex or trace.cubex";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;

public:
    POPHybridTransferTestAdd( cube::CubeProxy* );

    virtual
    ~POPHybridTransferTestAdd()
    {
    };

    virtual
    double
    analyze( const cube::list_of_cnodes& cnodes,
             cube::LocationGroup*        _lg = nullptr ) const;

// ------ overview tests ---------

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false   );


    bool
    isActive() const;

    bool
    isIssue() const;
};
};
#endif // POPHybrid_IMBALANCE_TEST_H
