/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_POP_HYBRID_OMP_REGION_TEST_H
#define CUBELIB_POP_HYBRID_OMP_REGION_TEST_H

#include <string>
#include "PerformanceTest.h"


namespace hybanalysis
{
class POPHybridThreadEfficiencyTest;
class POPHybridOmpRegionEfficiencyTest : public popcalculation::PerformanceTest
{
    friend class POPHybridThreadEfficiencyTest;

private:
    cube::Metric* pop_max_omp_comp;
    cube::Metric* pop_avg_omp_comp;
    cube::Metric* pop_avg_ser_comp;

    cube::list_of_metrics lavg_omp_metrics;
    cube::list_of_metrics lavg_ser_metrics;

protected:

    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "OpenMP Region Efficiency indicates inefficiencies within threads, and can be computed as follows: \n"
            "\tOpenMP Region Efficiency = \n "
            " (avg(time in useful computation within OpenMP ) + avg( serial computation ))/(avg( time in OpenMP ) + avg( serial computation ))\n"
            "Where average time in OpenMP and average serial computation are computed as weighted arithmetic mean. \n"
            "If number of threads is equal across processes average time in OpenMP and average serial computation \n"
            "can be computed as ordinary arithmetic mean.";
        std::string not_active_text =
            "OpenMP Region Efficiency metric is a basic POP metric and is available for every Score-P/Scalasca measurement. \n"
            "If Cube Report was produced by another tool than Score-P/Scalasca, it might have missing metric Time.\n"
            "In this case POP analysis is not possible.";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;

public:
    POPHybridOmpRegionEfficiencyTest( cube::CubeProxy* );

    virtual
    ~POPHybridOmpRegionEfficiencyTest()
    {
    };

    virtual
    double
    analyze( const cube::list_of_cnodes& cnodes,
             cube::LocationGroup*        _lg = nullptr ) const;

// ------ overview tests ---------

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false  );


    bool
    isActive() const;

    bool
    isIssue() const;
};
};
#endif // POPHybrid_IMBALANCE_TEST_H
