/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2023                                                     **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef FILESERVER_PLUGIN_H
#define FILESERVER_PLUGIN_H

#include <string>
#include "CubeServerPlugin.h"

namespace cube
{
class CubeProxy;
}

namespace fileserver_plugin
{
/**
 * @brief The FileServerPlugin class
 * This server side plugin allows to retreive the contents of a file on cube_server.
 * Client usage:
 *   PluginServices::sendToPlugin( "FileServerPlugin", <filename> )
 *   returns contents of the file as std::vector<unsigned char>
 *
 */
class FileServerPlugin : public cube::CubeServerPlugin
{
public:
    virtual const char*
    name() const override;

    std::vector<unsigned char>
    send( const std::vector<unsigned char>& filedata,
          cube::CubeProxy*                  cube ) override;
};

// the class factories

extern "C" cube::CubeServerPlugin * create()
{
    return new FileServerPlugin();
}

extern "C" void
destroy( cube::CubeServerPlugin* plugin )
{
    delete plugin;
}
}

#endif
